import {Component, OnDestroy, OnInit, ViewChild} from "@angular/core";
import {StakeholderService} from "../openaireLibrary/monitor/services/stakeholder.service";
import {EnvProperties} from "../openaireLibrary/utils/properties/env-properties";
import {Stakeholder, StakeholderEntities, Visibility} from "../openaireLibrary/monitor/entities/stakeholder";
import {Subscriber, zip} from "rxjs";
import {StakeholderUtils} from "../utils/indicator-utils";
import {UntypedFormBuilder, UntypedFormGroup} from "@angular/forms";
import {AlertModal} from "../openaireLibrary/utils/modal/alert";
import {Option} from "../openaireLibrary/sharedComponents/input/input.component";
import {Title} from "@angular/platform-browser";
import {UserManagementService} from "../openaireLibrary/services/user-management.service";
import {Session} from "../openaireLibrary/login/utils/helper.class";
import {EditStakeholderComponent} from "../general/edit-stakeholder/edit-stakeholder.component";
import {properties} from "../../environments/environment";
import {ActivatedRoute} from "@angular/router";

type Tab = 'all' | 'templates'| 'profiles';

declare var UIkit;

@Component({
  selector: 'home',
  templateUrl: "./manageStakeholders.component.html"
})
export class ManageStakeholdersComponent implements OnInit, OnDestroy {
  
  public properties: EnvProperties;
  public loading: boolean = true;
	public deleteLoading: boolean = false;
  public stakeholderUtils: StakeholderUtils = new StakeholderUtils();
  public defaultStakeholders: Stakeholder[];
  public stakeholders: Stakeholder[];
  public alias: string[];
  public stakeholder: Stakeholder;
  public index: number;
  public user = null;
  public tab: Tab = 'all';
	public stakeholderEntities = StakeholderEntities;
  /**
   *  Filtered Stakeholders
   */
  public displayDefaultStakeholders: Stakeholder[];
  public displayStakeholders: Stakeholder[];
  /**
   * Top filters
   */
  public filters: UntypedFormGroup;
  public all: Option = {
    value: 'all',
    label: 'All'
  };
  
  public callback: Function;
  
  /**
   * Grid or List View
   */
  private subscriptions: any[] = [];
  @ViewChild('editStakeholderModal', { static: true }) editStakeholderModal: AlertModal;
  @ViewChild('deleteStakeholderModal', { static: true }) deleteStakeholderModal: AlertModal;
  @ViewChild('editStakeholderComponent', { static: true }) editStakeholderComponent: EditStakeholderComponent;
  
  constructor(private stakeholderService: StakeholderService,
              private userManagementService: UserManagementService,
              private route: ActivatedRoute,
              private title: Title,
              private fb: UntypedFormBuilder) {
  }
  
  ngOnInit(): void {
    this.buildFilters();
    this.properties = properties;
    this.title.setTitle('Manage profiles');
    this.subscriptions.push(this.userManagementService.getUserInfo().subscribe(user => {
      this.user = user;
    }));
    let data = zip(
      this.stakeholderService.getDefaultStakeholders(this.properties.monitorServiceAPIURL),
      this.stakeholderService.getMyStakeholders(this.properties.monitorServiceAPIURL),
      this.stakeholderService.getAlias(this.properties.monitorServiceAPIURL)
    );
    this.subscriptions.push(data.subscribe(res => {
      this.defaultStakeholders = res[0];
      this.stakeholders = res[1];
      this.displayDefaultStakeholders = res[0];
      this.displayStakeholders = res[1];
      this.alias = res[2];
      this.loading = false;
    }, error => {
      this.loading = false;
    }));
    
    this.subscriptions.push(UIkit.util.on(document, 'hidden', '#edit_modal', (): void => {
      this.editStakeholderComponent.removePhoto();
    }));
  }
  
  ngOnDestroy(): void {
    this.subscriptions.forEach(value => {
      if (value instanceof Subscriber) {
        value.unsubscribe();
      } else if (value instanceof Function) {
        value();
      }
    });
  }
  
  hide(element: any) {
    UIkit.dropdown(element).hide();
  }
  
  
  private buildFilters() {
    this.filters = this.fb.group({
      status: this.fb.control('all'),
      keyword: this.fb.control('')
    });
    this.subscriptions.push(this.filters.get('status').valueChanges.subscribe(value => {
      this.onStatusChange(value);
    }));
    this.subscriptions.push(this.filters.get('keyword').valueChanges.subscribe(value => {
      this.onKeywordChange(value);
    }));
  }
  
  onStatusChange(value) {
    this.displayDefaultStakeholders = this.filterStatus(this.defaultStakeholders, value);
    this.displayStakeholders = this.filterStatus(this.stakeholders, value);
  }
  
  onKeywordChange(value) {
    this.displayDefaultStakeholders = this.filterByKeyword(this.defaultStakeholders, value);
    this.displayStakeholders = this.filterByKeyword(this.stakeholders, value);
  }
  
  private filterStatus(stakeholders: Stakeholder[], value): Stakeholder[] {
    if (value === 'all') {
      return stakeholders;
    } else {
      return stakeholders.filter(stakeholder => stakeholder.visibility == value);
    }
  }
  
  private filterByKeyword(stakeholders: Stakeholder[], value): Stakeholder[] {
    if (!value) {
      return stakeholders;
    } else {
      return stakeholders.filter(stakeholder =>
        stakeholder.name && stakeholder.name.toLowerCase().includes(value.toLowerCase()) ||
        stakeholder.type && stakeholder.type.toLowerCase().includes(value.toLowerCase()) ||
        stakeholder.index_id && stakeholder.index_id.toLowerCase().includes(value.toLowerCase()) ||
        stakeholder.index_shortName && stakeholder.index_shortName.toLowerCase().includes(value.toLowerCase()) ||
        stakeholder.index_name && stakeholder.index_name.toLowerCase().includes(value.toLowerCase())
      );
    }
  }
  
  public editStakeholder(stakeholder: Stakeholder = null, isDefault: boolean = false) {
    if (isDefault) {
      this.index = (stakeholder) ? this.defaultStakeholders.findIndex(value => value._id === stakeholder._id) : -1;
    } else {
      this.index = (stakeholder) ? this.stakeholders.findIndex(value => value._id === stakeholder._id) : -1;
    }
    if (!stakeholder) {
      this.stakeholder = new Stakeholder(null, null, null,
        null, null, null, null, null);
    } else {
      this.stakeholder = stakeholder;
    }
    this.editStakeholderComponent.init(this.stakeholder, this.alias, this.defaultStakeholders, isDefault, this.index === -1);
    if (this.index !== -1) {
      this.callback = (stakeholder: Stakeholder) => {
        let index: number;
        if (stakeholder.defaultId == null) {
          index = this.alias.findIndex(value => value == this.defaultStakeholders[this.index].alias);
          this.defaultStakeholders[this.index] = stakeholder;
        } else {
          index = this.alias.findIndex(value => value == this.stakeholders[this.index].alias);
          this.stakeholders[this.index] = stakeholder;
        }
        if(index != -1) {
          this.alias[index] = stakeholder.alias;
        }
				this.editStakeholderModal.cancel();
      };
      this.editStakeholderModal.alertTitle = 'Edit ' + this.stakeholder.name;
      this.editStakeholderModal.okButtonText = 'Save Changes';
    } else {
      this.callback = (stakeholder: Stakeholder) => {
        if (stakeholder.defaultId === null) {
          this.defaultStakeholders.push(stakeholder);
        } else {
          this.stakeholders.push(stakeholder);
        }
        this.alias.push(stakeholder.alias);
				this.editStakeholderModal.cancel();
      };
      this.editStakeholderModal.alertTitle = 'Create a new ' + (isDefault?'Default ':'') + 'Profile';
      this.editStakeholderModal.okButtonText = 'Create';
    }
    this.editStakeholderModal.cancelButtonText = 'Cancel';
    this.editStakeholderModal.okButtonLeft = false;
    this.editStakeholderModal.alertMessage = false;
		this.editStakeholderModal.stayOpen = true;
    this.editStakeholderModal.open();
  }
  
  public deleteStakeholderOpen(stakeholder: Stakeholder) {
    this.stakeholder = stakeholder;
    this.deleteStakeholderModal.alertTitle = 'Delete ' + this.stakeholder.index_name;
    this.deleteStakeholderModal.cancelButtonText = 'No';
    this.deleteStakeholderModal.okButtonText = 'Yes';
		this.deleteStakeholderModal.alertMessage = false;
		this.deleteStakeholderModal.stayOpen = true;
    this.deleteStakeholderModal.open();
  }
  
  public deleteStakeholder() {
		this.deleteLoading = true;
    if (!this.stakeholder.defaultId) {
      this.index = (this.stakeholder) ? this.defaultStakeholders.findIndex(value => value._id === this.stakeholder._id) : -1;
    } else {
      this.index = (this.stakeholder) ? this.stakeholders.findIndex(value => value._id === this.stakeholder._id) : -1;
    }
    this.subscriptions.push(this.stakeholderService.deleteElement(this.properties.monitorServiceAPIURL, [this.stakeholder._id]).subscribe(() => {
      UIkit.notification(this.stakeholder.name+ ' has been <b>successfully deleted</b>', {
        status: 'success',
        timeout: 6000,
        pos: 'bottom-right'
      });
      if (!this.stakeholder.defaultId) {
        this.defaultStakeholders.splice(this.index, 1);
      } else {
        this.stakeholders.splice(this.index, 1);
      }
      this.alias = this.alias.filter(item => item !== this.stakeholder.alias);
			this.deleteLoading = false;
			this.deleteStakeholderModal.cancel();
    }, error => {
      UIkit.notification('An error has occurred. Please try again later', {
        status: 'danger',
        timeout: 6000,
        pos: 'bottom-right'
      });
			this.deleteLoading = false;
			this.deleteStakeholderModal.cancel();
    }));
  }
  
  changeStakeholderStatus(stakeholder: Stakeholder, visibility: Visibility) {
    let path = [
      stakeholder._id
    ];
    this.subscriptions.push(this.stakeholderService.changeVisibility(this.properties.monitorServiceAPIURL, path, visibility).subscribe(visibility => {
      stakeholder.visibility = visibility;
      UIkit.notification(stakeholder.name+ '\'s status has been <b>successfully changed</b> to ' + stakeholder.visibility.toLowerCase(), {
        status: 'success',
        timeout: 6000,
        pos: 'bottom-right'
      });
    }, error => {
      UIkit.notification('An error has occurred. Please try again later', {
        status: 'danger',
        timeout: 6000,
        pos: 'bottom-right'
      });
    }));
  }
  
  public isManager(): boolean {
    return this.isCurator() || (Session.isKindOfMonitorManager(this.user));
  }
  
  public isProfileManager(stakeholder: Stakeholder): boolean {
    return this.isCurator() || (Session.isManager(stakeholder.type, stakeholder.alias, this.user));
  }
  
  public isCurator(): boolean {
    return this.isAdmin() || Session.isMonitorCurator(this.user);
  }
  
  public isAdmin(): boolean {
    return Session.isPortalAdministrator(this.user);
  }
  
  private isTab(tab: Tab): boolean {
    switch (tab) {
      case "all":
        return true;
      case "profiles":
        return true;
      case "templates":
        return true;
      default:
        return false;
    }
  }
}
